#include <vector>
#include "kktdirect.h"
#include "kktsupport.h"

void KKT_modify_ordering_to_respect_constraints(KKT_INT n,
                                                const KKT_INT *colstart,
                                                const KKT_INT *rowindex,
                                                const char *constrained,
                                                int safely_handle_dense_nodes,
                                                KKT_INT *ordering)
{
   // if we're being unsafe, find and mark the dense nodes
   Marker dense(n);
   if(!safely_handle_dense_nodes){
      const KKT_INT dense_threshhold=std::min(n-2, std::max(16, (KKT_INT)(10*std::sqrt((double)n))));
      for(KKT_INT i=0; i<n; ++i){
         if(colstart[i+1]-colstart[i]>dense_threshhold){
            dense.set(i);
         }
      }
   }
   // first count number of unconstrained neighbours for each constrained node
   std::vector<KKT_INT> counter(n, 0);
   for(KKT_INT i=0; i<n; ++i){
      if(constrained[i]){
         counter[i]=1;
         for(KKT_INT j=colstart[i]; j<colstart[i+1]; ++j){
            if(!constrained[rowindex[j]] && !dense[rowindex[j]]){
               ++counter[i];
            }
         }
      }
   }
   // now try to match the original ordering, but only emit constrained nodes when allowed
   KKT_INT a=0;
   for(KKT_INT b=0; b<n; ++b){
      KKT_INT i=ordering[b];
      if(!constrained[i]){
         ordering[a]=i;
         ++a;
         if(!dense[i]){
            for(KKT_INT j=colstart[i]; j<colstart[i+1]; ++j){
               KKT_INT k=rowindex[j];
               if(constrained[k]){
                  --counter[k];
                  if(counter[k]==0){
                     ordering[a]=k;
                     ++a;
                  }
               }
            }
         }
      }else{
         --counter[i];
         if(counter[i]==0){
            ordering[a]=i;
            ++a;
         }
      }
   }
   assert(a==n);
}
